#ifndef __TROOT_H
#define __TROOT_H
#include <TROOT.h>
#endif

#ifndef __TTREE_H
#define __TTREE_H
#include <TTree.h>
#endif


#ifndef __STDLIB_H
#define __STDLIB_H
#include <stdlib.h>
#endif

#ifndef __STDIO_H
#define __STDIO_H
#include <stdio.h>
#endif

#ifndef __IOSTREAM_H
#define __IOSTREAM_H
#include <iostream>
#endif

#ifndef __FSTREAM_H
#define __FSTREAM_H
#include <fstream>
#endif

#ifndef __VECTOR_H
#define __VECTOR_H
#include <vector>
#endif

#ifndef __STRING_H
#define __STRING_H
#include <string>
#endif

#ifndef __TFILE_H
#define __TFILE_H
#include <TFile.h>
#endif




using namespace std;
//----------------------------------------------------------------- 
// class TTable is for storing information from 
// table produced by lise.xls stored in txt file:
//-----------------------------------------------------------------
class TTable : public TObject
{
 private:
  UInt_t           fA;        // projectile mass
  UInt_t           fZ;        // projectile Z
  UInt_t           fZtgt;     // target Z
  string           fOption;    // string to store where the information 
                               // come from (atima, ziegler, hubert ...)
  vector<Float_t>   fEnergy;             // energy in MeV/u
  vector<Float_t>   fRange;              // range in fZtgt material
  vector<Float_t>   fRangeStraggling;    // range straggling in fZtgt material
  vector<Float_t>   fStoppingPower;      //stopping power
 
  
 public:
  TTable();
  TTable(const char *file_name);        //constructor to open file and fill the table
  void Init(const char *file_name);     //initialize 
  ~TTable();
  UInt_t Size();
  Double_t *GetEnergy(UInt_t index, UInt_t n);         // fetches energy for  interpolation function
  Double_t *GetRange(UInt_t index, UInt_t n);          // fetches range for interpolation function
  Double_t *GetRangeStraggling(UInt_t index, UInt_t n);  // fetches Rage Straggling for interpolation function
  Double_t *GetStoppingPower(UInt_t index, UInt_t n);    // fetches stopping power for interpolation function
  // it is always an array of length "n"
  void Dump(UInt_t index);
  Int_t FindEnergy(Double_t energy);     //finds index of energy closest to the one specified
  Int_t FindRange(Double_t range);     //finds index of range closest to the one specified
  Int_t FindRangeStraggling(Double_t straggling);   //returns index of array for closest value to Range straggling
  Int_t FindStoppingPower(Double_t stoppingpower);   // returns index of array for closest value to StoppingPower

  TTable operator=(const TTable &rhs);
   
  ClassDef (TTable,1)
};
//-------   End of Declarations for TTable class ---------------
// ----- Declarations for TSpline class: ---------
class TSpline : public TObject
{
 private:
  TTable             fTable;      //table for isotope and target I am working with
  vector<Double_t>   fDataX;     // data used for interpolation  X 
  vector<Double_t>   fDataY;     // data used for interpolation  Y
  

 public:
  TSpline(const TTable &table);
  TSpline(TTable *table);
  ~TSpline();
  void PolInterpolate(const Double_t x, Double_t &y, Double_t &dy);
  // function doing the Polynomial interpolation ^^^^^^^^^^^^^^^^^
  Double_t EnergyLoss(Double_t energy, Double_t thickness);   //function calculates energy loss for projectile with energy
  Double_t RangeInMatter(Double_t energy);   // function returns Range of Atima isotope in matter in mg/cm2
  Double_t RangeStraggling(Double_t energy);  // function returns Range Straggling in matter in mg/cm2
  Double_t StoppingPower(Double_t energy);   // Function calculates Stopping Power
  
  
  ClassDef(TSpline, 1)    
};
//===================================================================
//  class TInteraction uses TTable and TSpline classes to calculate
//  energy loss, range in matter by method of polynomial interpolation 
//  from data stored in TTable objects and stored in a tree according
//  to target material in file fFile!
// 
//===================================================================
class TInteraction : public TObject
{
 private:
  TFile         * fFile;   //file where the data is stored
  // data is in format of a ROOT Tree 
  

 public:
  TInteraction(TFile *f);
  ~TInteraction();
  void CreateInteractionTree(UInt_t Ztgt, const char *data);   // creates a TTree with name "Target element" from data
  // and saves it to a fFile
  Double_t EnergyLoss(UInt_t Ap, UInt_t Zp, UInt_t Zt, Double_t thickness, Double_t Energy);  // returns energy loss in MeV/u
  Double_t RangeInMatter(UInt_t Ap, UInt_t Zp, UInt_t Zt, Double_t Energy);            // returns Range in mg/cm2
  Double_t RangeStraggling(UInt_t Ap, UInt_t Zp, UInt_t Zt, Double_t Energy);         // returns Range Straggling in mg/cm2
  Double_t StoppingPower(UInt_t Ap, UInt_t Zp, UInt_t Zt, Double_t Energy);     // return stoppin power
  Double_t EnergyStraggling(UInt_t Ap, UInt_t Zp, UInt_t Zt, Double_t thickness, Double_t Energy);   // return energy straggling in MeV/u

  ClassDef(TInteraction, 1)   
};
//============================================================================================================
string GetElementName(UInt_t z);                      
Int_t GetNentry(TTree *t, UInt_t index);
